/* 
	DasBlinkenBoard v1.0

	Copyright (c) 2009 by James Delaney 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.


*/
#include "stdlib.h" 
#include <avr/io.h>

#include <avr/interrupt.h>      // definitions for interrupts

#ifndef F_CPU
// define cpu clock speed if not defined
// why 1Mhz if the cpu is running at 8mhz? Because of all the interrupts
#define F_CPU 1000000
#endif
#include <util/delay.h>


volatile uint8_t led; //volatile uint8_t pwm[8];
volatile int8_t pwm[8]; //volatile int seed;
volatile int8_t direction[8];
//volatile double delay=0.0;

void ledOn(unsigned char pos)
{
	//PORTA &= ~_BV(0);
	if (pos>4)
	{
		PORTB &= ~_BV(7-pos);
		//PORTB = 0b00000001;
	}
	else
	{
		if (pos==4) { pos=7; }
		PORTA &= ~_BV(pos);
	}
}
void ledOff(unsigned char pos)
{
	//PORTA = 0b00000000;
	//PORTA |= _BV(0);

	if (pos>4)
	{
		PORTB |= _BV(7-pos);
		//PORTB = 0b00000001;
	}
	else
	{
		if (pos==4) { pos=7; }
		PORTA |= _BV(pos);
	}
}
void ledToggle(unsigned char pos)
{
	if (pos>4)
	{
		PORTB ^= _BV(7-pos);
		//PORTB = 0b00000001;
	}
	else
	{
		if (pos==4) { pos=7; }
		PORTA ^= _BV(pos);
	}
}

uint8_t another_random(void)
{
// source http://www.avrfreaks.net/index.php?name=PNphpBB2&file=viewtopic&p=511923
    //32 bit random number, from noinit sram
    static uint32_t random31 __attribute__((section(".noinit")));
    //temp storage for random number
    uint32_t temp;
    temp = random31<<1;
    //if bit31 set, or if init sram happens to be 0
    if((temp & 0x80000000)||(temp == 0)){
        //xor magic number (taps)
        temp ^= 0x20AA95B5;
    }
    //save back to random number for next time
    random31 = temp;
    //return lowest byte only
    return temp;
} 

void flame_setup()
{
 	pwm[0]=25; direction[0]=1; 
 	pwm[1]=50; direction[1]=2;
 	pwm[2]=75; direction[2]=4;
 	pwm[3]=100; direction[3]=8;
 	pwm[4]=25; direction[4]=1;
 	pwm[5]=50; direction[5]=2;
 	pwm[6]=75; direction[6]=4;
 	pwm[7]=100; direction[7]=8;
}
void flame()
{
	static int speed;
	int lop;
	int port;
	for (lop=0; lop<8; lop++)
	{
		pwm[lop]=pwm[lop]+direction[lop]; 
		if (pwm[lop]>=100) { pwm[lop]=100; direction[lop]=-direction[lop];  }
		if (pwm[lop]<=0) {   pwm[lop]=0;   direction[lop]=-direction[lop]; }
	}
	speed++;
	if (speed>10)
	{
		speed= another_random() %20+1; //rand() % 8;
		port= another_random()%8 ; //(rand() %4);
		direction[port]=speed;
		speed=0;
	}
}
void doflame()
{
	int x;
	for (x=0; x<50; x++) flame();
}
void twinkle_on()
{
	//randomly select 1 led to go up ...when it reaches up it goes down
	int port,lop;

	if ((another_random()%100)>80)
	{
		port= another_random()%8 ; //(rand() %4);
		pwm[port]=0;
	}
	for (lop=0; lop<8; lop++)
	{
		pwm[lop]=pwm[lop]+20;
		if (pwm[lop]>=100) { pwm[lop]=100; }
	}
}
void dotwinkle_on()
{
	int x;
	for (x=0; x<50; x++) twinkle_on();
}

void bounce()
{
	int x;
	for (x=0; x<8; x++)
	{
		pwm[x]=0;
		_delay_ms(1);
		pwm[x]=100;
	}
	for (x=6; x>0; x--) 
	{
		pwm[x]=0;
		_delay_ms(1);
		pwm[x]=100;
	}
}
void pwm_down(int position)
{
	int lop;
	for (lop=0; lop<100; lop+=2)
	{
		pwm[position]=lop;
		//_delay_ms(delay);
	}
	pwm[position]=100;
}
void pwm_up(int position)
{
	int lop;
	for (lop=100; lop>0; lop-=2)
	{
		pwm[position]=lop;
		_delay_ms(0.01);
	}
	pwm[position]=0;
}
void pwm_down_fast(int position)
{
	int lop;
	for (lop=0; lop<100; lop+=5)
	{
		pwm[position]=lop;
		//_delay_ms(delay);
	}
	pwm[position]=100;
}
void pwm_up_fast(int position)
{
	int lop;
	for (lop=100; lop>0; lop-=5)
	{
		pwm[position]=lop;
		_delay_ms(0.01);
	}
	pwm[position]=0;
}
void pwm_up_down(int position1,int position2)
{
	int lop;
	for (lop=100; lop>0; lop-=2)
	{
		pwm[position1]=lop;
		pwm[position2]=100-lop;
		//_delay_ms(0.1);
	}
	pwm[position1]=0;
	pwm[position2]=100;
}
void all_off()
{
	int lop;
	for (lop=0; lop<8; lop++)
	{
		pwm[lop]=100;
	}
}
void all_fade_off_right()
{
	int lop,power;
	for (power=0; power<100; power+=4)
	{
		for (lop=0; lop<4; lop++)
		{
			pwm[lop]=power;
		}
	}
	all_off();
}
void all_fade_off_left()
{
	int lop,power;
	for (power=0; power<100; power+=4)
	{
		for (lop=4; lop<8; lop++)
		{
			pwm[lop]=power;
		}
	}
	all_off();
}
void binarycount()
{
	static int8_t count=0;
	count++;
	if (count &0b00000001)   pwm_up_fast(0); else pwm_down_fast(0);
	if (count &0b00000010)   pwm_up_fast(1); else pwm_down_fast(1);
	if (count &0b00000100)   pwm_up_fast(2); else pwm_down_fast(2);
	if (count &0b00001000)   pwm_up_fast(3); else pwm_down_fast(3);
	if (count &0b00010000)   pwm_up_fast(4); else pwm_down_fast(4);
	if (count &0b00100000)   pwm_up_fast(5); else pwm_down_fast(5);
	if (count &0b01000000)   pwm_up_fast(6); else pwm_down_fast(6);
	if (count &0b10000000)   pwm_up_fast(7); else pwm_down_fast(7);
}
void pwmchase()
{
	pwm_up_down(0,7);
	pwm_up_down(1,0);
	pwm_up_down(2,1);
	pwm_up_down(3,2);
	pwm_up_down(4,3);
	pwm_up_down(5,4);
	pwm_up_down(6,5);
	pwm_up_down(7,6);
}
void pwmchase2()
{
	pwm_up_down(0,6);
	pwm_up_down(1,7);
	pwm_up_down(2,0);
	pwm_up_down(3,1);
	pwm_up_down(4,2);
	pwm_up_down(5,3);
	pwm_up_down(6,4);
	pwm_up_down(7,5);
}
void pwmchase3()
{
	pwm[0]=0; pwm[7]=75; pwm[6]=95; pwm[5]=100; _delay_ms(1);
	pwm[1]=0; pwm[0]=75; pwm[7]=95; pwm[6]=100; _delay_ms(1);
	pwm[2]=0; pwm[1]=75; pwm[0]=95; pwm[7]=100; _delay_ms(1);
	pwm[3]=0; pwm[2]=75; pwm[1]=95; pwm[0]=100; _delay_ms(1);
	pwm[4]=0; pwm[3]=75; pwm[2]=95; pwm[1]=100; _delay_ms(1);
	pwm[5]=0; pwm[4]=75; pwm[3]=95; pwm[2]=100; _delay_ms(1);
	pwm[6]=0; pwm[5]=75; pwm[4]=95; pwm[3]=100; _delay_ms(1);
	pwm[7]=0; pwm[6]=75; pwm[5]=95; pwm[4]=100; _delay_ms(1);
}
void pwmbounce()
{
	pwm_up_down(0,1);
	pwm_up_down(1,0);
	pwm_up_down(2,1);
	pwm_up_down(3,2);
	pwm_up_down(4,3);
	pwm_up_down(5,4);
	pwm_up_down(6,5);
	pwm_up_down(7,6);
	pwm_up_down(6,7);
	pwm_up_down(5,6);
	pwm_up_down(4,5);
	pwm_up_down(3,4);
	pwm_up_down(2,3);
	pwm_up_down(1,2);
}
void ttl_right()
{
	pwm_up(3);
	pwm_up(2);
	pwm_up(1);
	pwm_up(0);
	_delay_ms(2);
	all_fade_off_right();
}
void ttl_left()
{
	pwm_up(4);
	pwm_up(5);
	pwm_up(6);
	pwm_up(7);
	_delay_ms(2);
	all_fade_off_left();
}
void post()
{
	int x;
	for (x=0; x<8; x++)
	{
		pwm[x]=0;
		_delay_ms(1);
		pwm[x]=100;
		_delay_ms(.5);
		pwm[x]=0;
		_delay_ms(1);
		pwm[x]=100;
		_delay_ms(.25);
	}
}
ISR (TIM0_COMPA_vect)
{
	led=led+4;
	if (led>100) { led=0; }
	// very important here to do PORTA as 0b01110000 it will
	// turn off all LEDS and Keep those internal pullups on.
	// if you turn off the internal pullups the jumpers
	// start reading noise.
	if (led==0) { PORTA&=0b01110000; PORTB=0b00000000; }
  	if (led>pwm[0]) ledOff(0);
  	if (led>pwm[1]) ledOff(1);
  	if (led>pwm[2]) ledOff(2);
  	if (led>pwm[3]) ledOff(3);
  	if (led>pwm[4]) ledOff(4);
  	if (led>pwm[5]) ledOff(5);
  	if (led>pwm[6]) ledOff(6);
  	if (led>pwm[7]) ledOff(7);
	TCNT0=0;
}

int main (void)
{
	int speed=24;
	int pattern=0;

	int8_t switches=0;

	// Set the port directions
	//////////76543210
	DDRA =  0b10001111;  // 1 = output 0=input
	DDRB =  0b00000111;

	// enable disable pullups
	PORTB = 0b00000000;
	PORTA = 0b01110000; // 0 = no pullup 1=pullup

 	// setup for timer compare interrupt
 	TCCR0A = 0b00000000; // /no timer compare match output
 	TCCR0B = 0b00000001; // force output compa 7, 321 no prescales
 	OCR0A  = 0b00011111; // ocra compared to tcnt0
 	TIMSK0 = 0b00000010; // timer counter 0 interrrupts  1 ocr0a


	all_off(); // turn all pwm off
	sei(); // enable interrupts
	//flame_setup();
	post();
	while (1)
	{
		speed++;
		// is it time to change random pattern
		if (speed>5) { speed=0; pattern=another_random() % 8; speed=another_random() % 10 ; }

		// read the switches
		switches =(( (~PINA) & 0b01110000) >>4);
		if (switches!=0) { pattern=switches; }

		//pattern=0; 
		switch(pattern)
		{
			//case 0: post(); break; // post is too boring
			case 0: pwmchase3(); break;
			case 1: doflame(); break;
			case 2: bounce(); break;
			case 3: pwmbounce(); break;
			case 4: ttl_right(); ttl_right(); ttl_left(); ttl_left(); speed+=4; break;
			case 5: twinkle_on(); break; //twinkle_off(); break;
			case 6: pwmchase2(); break;
			case 7: binarycount(); break;
		}
	}
}


